<?php

namespace Teknisa\PLA\Modules\Parametros\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

class ExportaVigencia extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $dataSourceName;
	protected $parameterBagColumns;
    protected $dataSourceManager;
    protected $util;
    protected $connection;
    protected $RestrContrService;
    protected $errors = array(
        'Vigencia'=> array(),
        'Servico' => array(),
        'Prato'   => array(),
        'Grupo'   => array()
    );
    protected $chaves;
    protected $nameProvider;
    protected $tabelas = array(
        'VIGFREQPRATMES' => array(
            'Servico' => 'FREQPRATMES',
            'Prato'   => 'ITFREQPRATMES',
            'Anexo'   => 'ANXFREQPRATMES'
        ),
        'VIGFREQOPCAGR' => array(
            'Servico' => 'FREQOPCAGR',
            'Prato'   => 'ITFREQOPCAGR',
            'Anexo'   => 'ANXFREQOPCAGR'
        ),
        'VIGRESTPRODGR' => array(
            'Servico' => 'RESTPRODGR',
            'Prato'   => 'RESTPRODGR',
            'Anexo'   => 'ANXRESTPRODGR'
        ),
        'VIGLIMPESERGR' => array(
            'Servico' => 'LIMPESERGR',
            'Prato'   => 'ITLIMPESERGR',
            'Anexo'   => 'ANXLIMPESERGR'
        ),
        'VIGFREQPRODGRPRAT' => array(
            'Servico' => 'FREQPRODGRPRAT',
            'Prato'   => 'ITFREQPRODGRPRAT',
            'Anexo'   => 'ANXFREQPRODGRPRAT'
        ),
        'VIGFREQGRPRATMES' => array(
            'Servico' => 'FREQGRPRATMES',
            'Prato'   => 'ITFREQGRPRATMES',
            'Anexo'   => 'ANXFREQGRPRATMES'
        ),
    );
    
    public function __construct($crudService, $parameterBag, $dataSourceManager, $util, $connection, $RestrContrService, $nameProvider, $fileServerURL, $zhFileUploader, $uploadRoutes) {
        $this->crudService       = $crudService;
        $this->parameterBag      = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->util              = $util;
        $this->connection        = $connection;
        $this->RestrContrService = $RestrContrService;
        $this->nameProvider      = $nameProvider;
        $this->fileServerURL     = $fileServerURL;
        $this->zhFileUploader    = $zhFileUploader;
        $this->uploadRoutes      = $uploadRoutes;
        $this->chaves            = array();
    }
    
    //Cascade save Vigencia->Servico->Prato
    public function saveVigencia($vigencia, $response, $dataSourceName) {
        try {
            $vigencia["NRVIGENCIA"] = $this->RestrContrService->getNewCode($dataSourceName);
            $dataSourceName = strtolower($dataSourceName);
            unset($vigencia['__new_code_pk']);   
            $dataSet = new DataSet($dataSourceName, array($vigencia));
            $request = new DTO\Request\DataSet($dataSet, "", "", "");
            $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
            $nrvigencia = $rowsToSave[0]["NRVIGENCIA"];
            $dataSet = new DataSet($dataSourceName, $rowsToSave);
            $this->dataSourceManager->persist($dataSet);
            return $nrvigencia;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function importarServicos($row) {
        if($row['DataSourceDestino'] == 'VIGRESTPRODGR'){
            $this->importaRestrProd($row);
        }else{
            $servicos = $this->RestrContrService->retornaServicosPorFilial($row['NRVIGENCIAORIG'], $row['CDFILIALORIG'], $this->tabelas[$row['DataSourceDestino']]['Servico']);
            $nrvigenciadest =  $row["NRVIGENCIA"];
            $tpoper = $row["TPOPER"];
    
            if ($tpoper == "S") {
                $this->deleteServByNrvigencia($nrvigenciadest, $this->tabelas[$row['DataSourceDestino']]['Servico']);
            }
            
            foreach ($servicos as $servico) {
                try {
                    $hasArvorePadrao = false;
                    $newCodePk = $this->saveServicoImport($servico, $row, $nrvigenciadest);
                    $itens = $this->getItfreq($servico["NR".$this->tabelas[$row['DataSourceOrigem']]['Servico']], $row);
                    $itensToSave = array();
                    foreach ($itens as &$item) {
                        if(!isset($item['CDPRATO']) || (isset($item['CDPRATO']) && $item['CDFILAUXPRAT'] == 'P')){
                            $canPush = true;
                            $item["NR".$this->tabelas[$row['DataSourceDestino']]['Servico']] = $newCodePk;
                            $item["NR".$this->tabelas[$row['DataSourceDestino']]['Prato']] = $this->RestrContrService->getNewCode($this->tabelas[$row['DataSourceDestino']]['Prato']);
                            $item["__is_new"] = true;
                            $item["NRVIGENCIA"] = $nrvigenciadest;
                            $item['CDSERVFILI'] = $servico['CDSERVFILI'];
                            if(isset($item['NRGRUPOPRATOSERV'])){
                                $grupoprato = $this->RestrContrService->retornaItemNr($row, $item, $this->tabelas[$row['DataSourceOrigem']]['Prato']);
                                if($grupoprato['NRGRUPOPRATOSERV'] == null){
                                    $canPush = false;
                                }else{
                                    $item['NRGRUPOPRATOSERV'] = $grupoprato['NRGRUPOPRATOSERV'];
                                }
                            }
                            if($canPush){
                                array_push($itensToSave, $item);
                            }else{
                                if(!isset($this->errors['Grupo'][$grupoprato['CDGRUPOPRATO'].' | '.$grupoprato['NMGRUPOPRATO']])){
                                    $this->errors['Grupo'][$grupoprato['CDGRUPOPRATO'].' | '.$grupoprato['NMGRUPOPRATO']] = array();
                                    array_push($this->errors['Grupo'][$grupoprato['CDGRUPOPRATO'].' | '.$grupoprato['NMGRUPOPRATO']], $servico['CDSERVFILI'].' | '.$servico['NMSERVICO'].'_'.$row['CDFILIAL'].' | '.$row['NMFILIAL']);
                                }else{
                                    array_push($this->errors['Grupo'][$grupoprato['CDGRUPOPRATO'].' | '.$grupoprato['NMGRUPOPRATO']], $servico['CDSERVFILI'].' | '.$servico['NMSERVICO'].'_'.$row['CDFILIAL'].' | '.$row['NMFILIAL']);
                                }
                            }
                        }else{
                            $hasArvorePadrao = true;
                        }
                    }
                    if($hasArvorePadrao){
                        array_push($this->errors['Prato'], $row['CDFILIAL'].' | '.$row['NMFILIAL']);
                    }
                    $dataSourceName = strtolower($this->tabelas[$row['DataSourceDestino']]['Prato']);
                    $dataSet = new DataSet($dataSourceName, $itensToSave);
                    $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
                    $rowsToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
                    $dataSet = new DataSet($dataSourceName, $rowsToSave);
                    $this->dataSourceManager->persist($dataSet);
                } catch (\Exception $e) {
                    if(strpos($e->getMessage(), 'ORA-02291')){
                        if(isset($this->errors['Servico'][$servico['CDSERVFILI'].' | '.$servico["NMSERVICO"]])){
                            array_push($this->errors['Servico'][$servico['CDSERVFILI'].' | '.$servico["NMSERVICO"]], $row['CDFILIAL'].' | '.$row['NMFILIAL']);
                        }else{
                            $this->errors['Servico'][$servico['CDSERVFILI'].' | '.$servico["NMSERVICO"]] = array();
                            array_push($this->errors['Servico'][$servico['CDSERVFILI'].' | '.$servico["NMSERVICO"]], $row['CDFILIAL'].' | '.$row['NMFILIAL']);
                        }
                    }else{
                        throw $e;
                    }
                }
            }
        }
    }

    private function saveServicoImport($servico, $vigencia, $nrvigenciadest) {
        $dataSourceName = $this->tabelas[$vigencia['DataSourceDestino']]['Servico'];

        $servico["NRVIGENCIA"]         = $nrvigenciadest;
        $servico["DTVIGENCIA"]         = $vigencia["DTVIGENCIA"];
        $servico["CDFILIAL"]           = $vigencia["CDFILIAL"];
        $servico["DS".$dataSourceName] = isset($servico["DS".$this->tabelas[$vigencia['DataSourceOrigem']]['Servico']]) ? $servico["DS".$this->tabelas[$vigencia['DataSourceOrigem']]['Servico']] : '';
        $primaryKey                    = $this->RestrContrService->getNewCode($dataSourceName);
        $servico["NR".$dataSourceName] = $primaryKey;
        $servico["__is_new"]           = true;
        $dataSourceName                = strtolower($dataSourceName);
        
        $dataSet = new DataSet($dataSourceName, array($servico));
        $request = new DTO\Request\DataSet($dataSet, "", "", "");
        $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
        $dataSet = new DataSet($dataSourceName, $rowsToSave);
        $this->dataSourceManager->persist($dataSet);
        return $primaryKey;
    }

    public function getItfreq($nrfreqpratmes, $row) {
        $conditions = $this->RestrContrService->buildConditions(array('NR'.$this->tabelas[$row['DataSourceOrigem']]['Servico']), array($nrfreqpratmes));
        return $this->crudService->getFilteredDataSet(strtolower($this->tabelas[$row['DataSourceOrigem']]['Prato']), $conditions)->getRows();
    }

    private function deleteServByNrvigencia($nrvigencia, $tableName) {
        $sql = "DELETE FROM ".$tableName." WHERE NRVIGENCIA = :NRVIGENCIA AND NRORG = :NRORG";
        $params = array(
            "NRVIGENCIA"  => $nrvigencia,
            "NRORG"       => $this->crudService->getEnvironment()->getNrOrg()
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }

    public function vigenciaExport(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $vigenciaOrig = $row["VIGENCIAORIG"];
            $tables = $row["VIGENCIADEST"];
            $unidades = $row["UNIDDEST"];
            $hasAnexo = $vigenciaOrig['ANEXO'];
            foreach($tables as $table) {
                if (in_array(strtoupper($table), $this->RestrContrService->getTables())) { // Prevent SQL injection
                    foreach($unidades as $unidade){
                        $row = array(
                            "CDFILIAL"         => $unidade['CDFILIAL'],
                            "NMFILIAL"         => $unidade['NMFILIAL'],
                            "CDFILIALORIG"     => $vigenciaOrig['CDFILIAL'],
                            "DTVIGENCIA"       => $vigenciaOrig["DTVIGENCIA"],
                            "DTVIGENCIAFIM"    => $vigenciaOrig["DTVIGENCIAFIM"],
                            "DESCRICAO"        => $vigenciaOrig["DESCRICAO"],
                            "IDTIPORESTRICAO"  => $vigenciaOrig["IDTIPORESTRICAO"],
                            "NRVIGENCIAORIG"   => $vigenciaOrig['NRVIGENCIA'],
                            "CDDESCOBSFILIAL"  => isset($vigenciaOrig['CDDESCOBSFILIAL']) ? $vigenciaOrig['CDDESCOBSFILIAL'] : "",
                            "TPOPER"           => $vigenciaOrig['TPOPER'],
                            "dataSourceName"   => strtolower($table),
                            "DataSourceDestino"=> $table,
                            "DataSourceOrigem" => $vigenciaOrig['DataSourceOrigem'],
                            "fieldsToValidate" => array("CDFILIAL", "DTVIGENCIA","IDTIPORESTRICAO"),
                            "__is_new"         => true
                        );
                        try {
                            $canExport = true;
                            if($vigenciaOrig['TPOPER'] == 'S'){
                                $canExport = $this->RestrContrService->apagaVigencia($row);
                            }else{
                                $this->RestrContrService->validateRegisterVigencia($row);
                            }
                            
                            if (isset($row["DTVIGENCIAFIM"]) && strlen($row["DTVIGENCIAFIM"])) {
                                $this->RestrContrService->validateIntervalOverlap(null, $row["DTVIGENCIA"], $row["DTVIGENCIAFIM"], $row["dataSourceName"], $row["CDFILIAL"], $row["IDTIPORESTRICAO"]);
                            }
                            if($canExport){
                                $row['NRVIGENCIA'] = $this->saveVigencia($row, $response, $row["DataSourceDestino"]);
                                $this->importarServicos($row);
                                if($hasAnexo == 'S'){
                                    $this->importarAnexos($row);
                                }
                            }
                        } catch( \Exception $e) {
                            if ($e->getMessage() == "registerExists" || $e->getMessage() == "intervalOverlap") {
                                array_push($this->errors['Vigencia'], $unidade['CDFILIAL'].' | '.$unidade['NMFILIAL']);
                            } else {
                                throw $e;
                            }
                        }
                    }
                }
            }
            
            if (!empty($this->errors['Vigencia']) || !empty($this->errors['Servico']) || !empty($this->errors['Grupo']) || !empty($this->errors['Prato'])) {
                $response->addDataSet(new DataSet("errors", $this->errors));
                $response->setError(new Error("unityExportErrors", 500));
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function verificaParaExportacao($request, $response){
        $conditions = array(
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->crudService->getEnvironment()->getNrOrg()
            ),
            array(
                "columnName" => "IDTIPORESTRICAO",
                "operator" => "=",
                "value" => $request->getRow()['IDTIPORESTRICAO']
            )
        );
        $dataSet = $this->crudService->getFilteredDataset("padgrupoprato", $conditions);
        $padraoPrato = $dataSet->getRows();
        
        $conditions = array(
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->crudService->getEnvironment()->getNrOrg()
            )
        );
        
        $dataSet = $this->crudService->getFilteredDataset("servpadr", $conditions);
        $padraoServico = $dataSet->getRows();
        
        $response->addDataSet(new DataSet('RESULT', array(sizeof($padraoPrato) && sizeof($padraoServico))));
    }
    
    public function importarAnexos($row){
        $conditions = array(
            array(
                "columnName" => "NRVIGENCIA",
                "operator" => "=",
                "value" => $row['NRVIGENCIAORIG']
            )
        );

        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria(strtolower($this->tabelas[$row['DataSourceOrigem']]['Anexo']), $conditions);
        $dataSetAnexos = $this->dataSourceManager->findBy($filterCriteria);
        $anexos = $dataSetAnexos->getRows();
        $nrvigenciadest =  $row["NRVIGENCIA"];
        $tpoper = $row["TPOPER"];
        try{
            $initialFiles = array();
            
            foreach($anexos as &$anexo){
                $fileRow = array();
                $anexo['NRANEXO'] = $this->RestrContrService->getNewCode(strtolower($this->tabelas[$row['DataSourceDestino']]['Anexo'])); 
                $anexo['NRVIGENCIA'] = $nrvigenciadest;
                $name = $anexo['DOCUMENTOANEXO'];
                $file = file_get_contents($this->fileServerURL.$name);
                $extensao = substr($anexo['DSDOCUMENTOANEXO'], strrpos($anexo['DSDOCUMENTOANEXO'], '.'));
                $type = 'data:application/wps-office'.$extensao;
                $finalFile = $type.';base64,'.base64_encode($file);
                $fileRow = array(
                    'fileUpload' => array(
                        array(
                            'b64File'            => $finalFile,
                            "name"               => $anexo['DSDOCUMENTOANEXO'],
                            'type'               => $type,
                            'webkitRelativePath' => '',
                            'size'               => rand(1000, 20000)
                        )
                    )
                );
                $this->zhFileUploader->uploadFilesInRow($fileRow, $this->uploadRoutes['/restr_contr_upload_anexo']['fileField'], $this->uploadRoutes['/restr_contr_upload_anexo']['customFolder']);
                $anexo['DOCUMENTOANEXO'] = $fileRow['fileUpload'][0]['path'];
                $anexo['__is_new'] = true;
            }
            $dataSet = new DataSet(strtolower($this->tabelas[$row['DataSourceDestino']]['Anexo']), $anexos);
            $this->dataSourceManager->persist($dataSet);
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function importaRestrProd($row){
        $servicos = $this->RestrContrService->retornaServicosRestrProd($row['NRVIGENCIAORIG'], $row['CDFILIALORIG']);
        $nrvigenciadest =  $row["NRVIGENCIA"];
        $tpoper = $row["TPOPER"];

        if ($tpoper == "S") {
            $this->deleteServByNrvigencia($nrvigenciadest, $this->tabelas[$row['DataSourceDestino']]['Servico']);
        }
        try {
            foreach ($servicos as $servico) {
                $grupoprato = $this->RestrContrService->retornaItemNr($row, $servico, $this->tabelas[$row['DataSourceOrigem']]['Prato']);
                if($grupoprato['NRGRUPOPRATOSERV'] != null){
                    $servico['NRGRUPOPRATOSERV'] = $grupoprato['NRGRUPOPRATOSERV'];
                    $this->saveServicoImport($servico, $row, $nrvigenciadest);
                }else{
                    if(!isset($this->errors['Grupo'][$grupoprato['CDGRUPOPRATO'].' | '.$grupoprato['NMGRUPOPRATO']])){
                        $this->errors['Grupo'][$grupoprato['CDGRUPOPRATO'].' | '.$grupoprato['NMGRUPOPRATO']] = array();
                        array_push($this->errors['Grupo'][$grupoprato['CDGRUPOPRATO'].' | '.$grupoprato['NMGRUPOPRATO']], $servico['NMSERVICO'].'_'.$row['CDFILIAL'].' | '.$row['NMFILIAL']);
                    }else{
                        array_push($this->errors['Grupo'][$grupoprato['CDGRUPOPRATO'].' | '.$grupoprato['NMGRUPOPRATO']], $servico['NMSERVICO'].'_'.$row['CDFILIAL'].' | '.$row['NMFILIAL']);
                    }
                }
            }
        } catch (\Exception $e) {
            if(strpos($e->getMessage(), 'ORA-02291')){
                if(isset($this->errors['Servico'][$servico['CDSERVFILI'].' | '.$servico["NMSERVICO"]])){
                    array_push($this->errors['Servico'][$servico['CDSERVFILI'].' | '.$servico["NMSERVICO"]], $row['CDFILIAL'].' | '.$row['NMFILIAL']);
                }else{
                    $this->errors['Servico'][$servico['CDSERVFILI'].' | '.$servico["NMSERVICO"]] = array();
                    array_push($this->errors['Servico'][$servico['CDSERVFILI'].' | '.$servico["NMSERVICO"]], $row['CDFILIAL'].' | '.$row['NMFILIAL']);
                }
            }else{
                throw $e;
            }
        }
    }
 }