<?php
namespace Teknisa\PLA\Modules\Parametros\Controllers;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;


class Pla02400SetorDoCardapio extends \Teknisa\GER\Controllers\TeknisaCrud {
	protected $dataSourceName;
	protected $parameterBagColumns;
	protected $dataSourceNameToSave;
	protected $crudService;
	protected $dataSourceManager;
	protected $parameterBag;
	protected $environment;
	protected $entityManager;
	protected $connection;

	public function __construct($crudService, $parameterBag, $dataSourceManager, $environment, $entityManager, $connection){
		$this->crudService 			 = $crudService;
		$this->parameterBag      = $parameterBag;
		$this->dataSourceManager = $dataSourceManager;
		$this->environment 			 = $environment;
		$this->entityManager = $entityManager;
		$this->connection = $connection;

		parent::__construct($crudService, $parameterBag);
	}

	public function findPadrao($request, $response){
		$this->dataSourceName = 'pla02400_padrao';
		$this->parameterBagColumns = array('CDFILIAL', 'NRORG');
		parent::find($request, $response);
	}

	public function savePadrao($request, $response){
		$this->dataSourceNameToSave = 'setotipoprat';
		parent::save($request, $response);
	}

	public function findExcecoes($request, $response){
		$this->dataSourceName = 'pla02400_excecoes';
		parent::find($request, $response);
	}


	public function saveExcecoes($request, $response){
		try {
			$row = $request->getDataSet()->getRows()[0];
			$conditions = array(
				array(
					'columnName' => 'CDFILIAL',
					'operator' => '=',
					'value' => $row['CDFILIAL']
				),
				array(
					'columnName' => 'CDTIPOPRAT',
					'operator' => '=',
					'value' => $row['CDTIPOPRAT']
				),
				array(
					'columnName' => 'CDPRODUTO',
					'operator' => '=',
					'value' => $row['CDPRODUTO']
				),
				array(
					'columnName' => 'CDSETORCOZ',
					'operator' => '=',
					'value' => $row['CDSETORCOZ']
				)
			);

			//$this->verifyExistenceDietapadr($row['CDDIETPAD'], $conditions);
			$filial = $row['CDFILIAL'];
			$tipoPrat = $row['CDTIPOPRAT'];
			$produto = $row['CDPRODUTO'];
			$dieta = $row['CDDIETPAD'];
			$verificaExcecaoProduto = $this->verificaExcecaoProduto($filial, $tipoPrat, $produto, $dieta);
			if($verificaExcecaoProduto !== false){
				throw new \Exception("Impossível adicionar. Registro já existe.");	
			}
			$row['NRSEQSETOPRC'] = $this->findNvlMaxExcecoes($conditions);
			$this->dataSourceNameToSave = 'setoprodcard';
			parent::save($request, $response);
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}

	public function verifyExistenceDietapadr($cddieta, $conditions){
		$dietapadr;
		if(empty($cddieta)){
			$dietapadr = array(
				'columnName' => 'CDDIETPAD',
				'operator' => 'IS NULL OR CDDIETPAD = ',
				'value' => NULL
			);
		} else {
			$dietapadr = array(
				'columnName' => 'CDDIETPAD',
				'operator' => '=',
				'value' => $cddieta
			);
		}
		array_push($conditions, $dietapadr);

		$filterCriteria = new FilterCriteria('setoprodcard', $conditions);
		$filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
		$rows = $this->dataSourceManager->findBy($filterCriteria)->getRows();

		array_pop($conditions);

		if(!empty($rows)){
			throw new \Exception("Impossível adicionar. Registro já existe.");
		}
	}

	public function findNvlMaxExcecoes($conditions){
		$filterCriteria = new FilterCriteria('pla02400_max_nrseqsetoprc', $conditions);
		$parameterBagColumns = array('CDFILIAL','CDSETORCOZ', 'CDTIPOPRAT', 'CDPRODUTO', 'NRORG');
		$filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
		$newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $parameterBagColumns);
		$rows = $this->dataSourceManager->findBy($newFilterCriteria)->getRows();
		if(empty($rows) && empty($rows[0])){
			return '01';
		} else {
			$sequencial = intval($rows[0]['NRSEQSETOPRC']) + 1;
			if($sequencial < 10) {
				return '0'.strval($sequencial);
			} return strval($sequencial);
		}
	}

	public function deleteExcecoes($request, $response){
		$this->dataSourceNameToSave = 'setoprodcard';
		$this->dataSourceName = 'setoprodcard';
		parent::delete($request, $response);
	}

	public function findExcecoesTransportadas($request, $response){
		$this->dataSourceName = 'pla02400_excecoes_transportadas';
		parent::find($request, $response);
	}
	
	public function saveExcecoesTransportadas($request, $response){
		$this->dataSourceNameToSave = 'setoprodcardtrans';
		parent::save($request, $response);
	}
	
	public function deleteExcecoesTransportadas($request, $response){
		$this->dataSourceName = 'setoprodcardtrans';
		parent::delete($request, $response);
	}
	
	public function verificaTipoPrato($filial, $tipoPrat){
		$sql = "SELECT CDTIPOPRAT FROM SETOTIPOPRAT WHERE CDFILIAL = :P_CDFILIAL AND CDTIPOPRAT = :P_CDTIPOPRAT";
		$params = array(
		    "P_CDFILIAL" => $filial,
		    "P_CDTIPOPRAT" => $tipoPrat
		);
		$query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetch();
        return $result;
	}
	
	public function confirmaCadastroPadrao(DTO\Request $request, DTO\Response $response){
		try{
			$row = $request->getRow();
			$filial = $row['CDFILIAL'];
			$tipoPratArray = $row['NMTIPOPRATCADASTRO'];
			foreach ($tipoPratArray as $tipoPrat){
				$verificaTipoPrato = $this->verificaTipoPrato($filial, $tipoPrat);
				if($verificaTipoPrato === false){
					$this->connection->beginTransaction();
					$insert = "INSERT INTO SETOTIPOPRAT (CDFILIAL, CDTIPOPRAT, CDSETORCOZ, NRORG, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO) VALUES (:P_CDFILIAL, :P_TIPOPRAT, :P_CDSETORCOZ, :P_NRORG, SYSDATE, :P_NRORG, :P_CDOPERADOR)";
					$params = array(
						"P_CDFILIAL"   => $filial,
						"P_TIPOPRAT"   => $tipoPrat,
						"P_CDSETORCOZ" => $row['CDSETORCOZ'],
						"P_NRORG"      => $row['NRORG'],
						"P_CDOPERADOR" => $this->environment->getCdOperador()
					);
					
					$queryInsert = $this->connection->prepare($insert);
					$queryInsert->execute($params);
					$this->connection->commit();
				}else{
					$this->connection->beginTransaction();
					$update = "UPDATE SETOTIPOPRAT ST SET ST.CDSETORCOZ = :P_CDSETORCOZ, ST.CDOPERULTATU = :P_CDOPERADOR, ST.DTULTATU = SYSDATE WHERE ST.CDTIPOPRAT = :P_CDTIPOPRAT AND ST.CDFILIAL = :P_CDFILIAL";
					$params = array(
						"P_CDSETORCOZ" => $row['CDSETORCOZ'],
						"P_CDOPERADOR" => $this->environment->getCdOperador(),
						"P_CDTIPOPRAT" => $tipoPrat,
						"P_CDFILIAL"   => $filial
					);
					
					$queryUpdate = $this->connection->prepare($update);
					$queryUpdate->execute($params);
					$this->connection->commit();
				}
			}
		}catch(\Exception $e){
            $this->connection->rollback();
            throw $e;
		}
	}
	
	public function verificaExcecaoProduto($filial, $tipoPrat, $produto, $dieta){
		$sql = "SELECT NRSEQSETOPRC, CDSETORCOZ FROM SETOPRODCARD WHERE ( CDFILIAL = :P_CDFILIAL ) AND ( CDPRODUTO = :P_CDPRODUTO ) AND ( CDTIPOPRAT = :P_CDTIPOPRAT ) AND ( ( ( :P_CDDIETPAD IS NULL ) AND ( CDDIETPAD IS NULL ) ) OR ( ( :P_CDDIETPAD IS NOT NULL ) AND ( CDDIETPAD = :P_CDDIETPAD ) ) )";
		$params = array(
		    "P_CDFILIAL"   => $filial,
		    "P_CDTIPOPRAT" => $tipoPrat,
		    "P_CDPRODUTO"  => $produto,
		    "P_CDDIETPAD"  => $dieta
		);
		$query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetch();
        return $result;
	}
	
	public function geraNrSeqSetoPrc ($filial, $tipoPrat, $produto, $setorCoz){
		$conditions = array(
			array(
				'columnName' => 'CDFILIAL',
				'operator' => '=',
				'value' => $filial
			),
			array(
				'columnName' => 'CDTIPOPRAT',
				'operator' => '=',
				'value' => $tipoPrat
			),
			array(
				'columnName' => 'CDPRODUTO',
				'operator' => '=',
				'value' => $produto
			),
			array(
				'columnName' => 'CDSETORCOZ',
				'operator' => '=',
				'value' => $setorCoz
			)
		);
		$nivelMaxExcecoes = $this->findNvlMaxExcecoes($conditions);
		return $nivelMaxExcecoes;
	}
	
	public function verificaSetorCozinha ($filial, $tipoPrat, $produto, $dieta, $setorCoz){
		$sql = "SELECT NRSEQSETOPRC, CDSETORCOZ FROM SETOPRODCARD WHERE ( CDFILIAL = :P_CDFILIAL ) AND ( CDPRODUTO = :P_CDPRODUTO ) AND ( CDTIPOPRAT = :P_CDTIPOPRAT ) AND ( ( ( :P_CDDIETPAD IS NULL ) AND ( CDDIETPAD IS NULL ) ) OR ( ( :P_CDDIETPAD IS NOT NULL ) AND ( CDDIETPAD = :P_CDDIETPAD ) ) ) AND ( CDSETORCOZ = :P_CDSETORCOZ )";
		$params = array(
		    "P_CDFILIAL"   => $filial,
		    "P_CDTIPOPRAT" => $tipoPrat,
		    "P_CDPRODUTO"  => $produto,
		    "P_CDDIETPAD"  => $dieta,
		    "P_CDSETORCOZ" => $setorCoz
		);
		$query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetch();
        return $result;
	}
	
	public function confirmaCadastroExcecoes(DTO\Request $request, DTO\Response $response){
		try{
			$row = $request->getRow();
			$filial = $row['CDFILIAL'];
			$tipoPratArray = $row['NMTIPOPRATCADASTRO'];
			$produtoArray = $row['CDPRODUTO'];
			$setorCoz = $row['CDSETORCOZ'];
			$dieta = $row['CDDIETPAD'];
			$this->connection->beginTransaction();
			foreach($produtoArray as $produto){
				foreach ($tipoPratArray as $tipoPrat){
					$verificaExcecaoProduto = $this->verificaExcecaoProduto($filial, $tipoPrat, $produto, $dieta);
					$nrSeqSetoPrc = $this->geraNrSeqSetoPrc($filial, $tipoPrat, $produto, $setorCoz);
					if($verificaExcecaoProduto === false){
						$insert = "INSERT INTO SETOPRODCARD (CDTIPOPRAT, CDFILIAL, CDSETORCOZ, CDPRODUTO, CDDIETPAD, NRSEQSETOPRC, NRORG, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO ) VALUES ( :P_TIPOPRAT, :P_CDFILIAL, :P_CDSETORCOZ, :P_CDPRODUTO, :P_CDDIETPAD, :P_NRSEQSETOPRC, :P_NRORG, SYSDATE, :P_NRORG, :P_CDOPERADOR )";
						$params = array(
							"P_TIPOPRAT"     => $tipoPrat,
							"P_CDFILIAL"     => $filial,
							"P_CDSETORCOZ"   => $setorCoz,
							"P_CDPRODUTO"    => $produto,
							"P_CDDIETPAD"    => $dieta,
							"P_NRSEQSETOPRC" => $nrSeqSetoPrc,
							"P_NRORG"        => $row['NRORG'],
							"P_CDOPERADOR"   => $this->environment->getCdOperador()
						);
					
						$queryInsert = $this->connection->prepare($insert);
						$queryInsert->execute($params);
					}else{
						$verificaSetorCozinha = $this->verificaSetorCozinha($filial, $tipoPrat, $produto, $dieta, $setorCoz);
						if($verificaSetorCozinha === false){
							$update = "UPDATE SETOPRODCARD SET CDSETORCOZ = :P_CDSETORCOZ, DTULTATU = SYSDATE, CDOPERULTATU = :P_CDOPERADOR WHERE ( CDFILIAL = :P_CDFILIAL ) AND ( CDPRODUTO = :P_CDPRODUTO ) AND ( CDTIPOPRAT = :P_CDTIPOPRAT ) AND ( ( ( :P_CDDIETPAD IS NULL ) AND ( CDDIETPAD IS NULL ) ) OR ( ( :P_CDDIETPAD IS NOT NULL ) AND ( CDDIETPAD = :P_CDDIETPAD ) ) ) AND (NRSEQSETOPRC = :P_NRSEQSETOPRC) AND (CDSETORCOZ = :P_CDSETORCOZORIG)";
							$params = array(
							"P_CDSETORCOZ"     => $setorCoz,
							"P_CDOPERADOR"     => $this->environment->getCdOperador(),
							"P_CDFILIAL"       => $filial,
							"P_CDPRODUTO"      => $produto,
							"P_CDTIPOPRAT"     => $tipoPrat,
							"P_CDDIETPAD"      => $dieta,
							"P_NRSEQSETOPRC"   => $verificaExcecaoProduto['NRSEQSETOPRC'],
							"P_CDSETORCOZORIG" => $verificaExcecaoProduto['CDSETORCOZ']
							);
							$queryUpdate = $this->connection->prepare($update);
							$queryUpdate->execute($params);
						}
					}
				}
			}
			$this->connection->commit();
		}catch(\Exception $e){
            $this->connection->rollback();
            throw $e;
		}
	}
}