<?php
namespace Teknisa\PLA\Modules\Relatorio\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\GER\Utilities\Environment;

class Pla08300HistoricoAjusteDeCardapio extends \Zeedhi\Framework\Controller\Report {
	
	protected $reportService;
	protected $crudService;
	protected $novoCodigo;
	protected $environment;
	protected $parameterBagColumns;
	protected $dataSourceManager;
	protected $conn;
	protected $birtStrategy;
	protected $utilPla;

	public function __construct($reportService,$crudService, $novoCodigo, $environment, $dataSourceManager, $conn, $qr2Strategy, $birtStrategy, $utilPla) {
		$this->reportService         	= $reportService;
		$this->crudService         		= $crudService;
		$this->novoCodigo         		= $novoCodigo;
		$this->environment       		= $environment;
		$this->dataSourceManager 	    = $dataSourceManager;
		$this->conn 					= $conn;
		$this->qr2Strategy				= $qr2Strategy;
		$this->birtStrategy 			= $birtStrategy;
		$this->utilPla					= $utilPla;
	}

	protected $reportMapping = array (
  	'PLA08300' => //"Histórico Ajuste de Cardápio"
     	array (
       	'strategy' => 'BIRT',
       	'parameterMapping' =>
	       	array (
	       	  	"DTMESCARD"		=>"P_DTMESCARD",
			 	"NRLISTA"		=>"P_NRLISTA",
			 	'CDPICTPRAT'	=>"P_CDPICTPRAT",
			 	"NMTIPOPRAT"	=>"P_CDTIPOPRAT",
			 	"CDGRUPFILI"	=>"P_CDGRUPFILI",
			 	"NMGRUPFILI"	=>"P_NMGRUPFILI",
			 	"NRORG" 		=>"P_NRORG"
				)
			),
  	'PLA08301' => //"Aderência de Cardápio (Percentual)"
     	array (
       	'strategy' => 'BIRT',
       	'parameterMapping' =>
	       	array (
	       	  	"DTMESCARD"		=>"P_DTMESCARD",
			 	"NRLISTA"		=>"P_NRLISTA",
			 	"VRPERCENT" 	=>"P_VRPERCENTUAL",
			 	"NRORG" 		=>"P_NRORG"
			)
		),
	'PLA08302' => //"Aderência de Cardápio por Tipo de Prato (Percentual)"
     	array (
      	'strategy' => 'BIRT',
      	'parameterMapping' =>
	      	array (
	        	"DTMESCARD"		=>"P_DTMESCARD",
				"NRLISTA"		=>"P_NRLISTA",
				"NMTIPOPRAT"	=>"P_CDTIPOPRAT",
				"CDGRUPFILI"	=>"P_CDGRUPFILI",
				"NRORG" 		=>"P_NRORG",
				"NMGRUPFILI"	=>"P_NMGRUPFILI"
			)
		),
	'PLA08303' => //"Alterações do Cardápio"
     	array (
      	'strategy' => 'BIRT',
      	'parameterMapping' =>
	      	array (
	        	"DTMESCARD"		=>	"P_DTMESCARD",
			 	"NRLISTA"		=>	"P_NRLISTA",
			 	"NMTIPOPRAT"	=>  "P_CDTIPOPRAT",
			 	'CDPICTPRAT'	=>  "P_CDPICTPRAT",
			 	'NRORG'     	=>	"P_NRORG"
			)
		),
		'PLA08304' => //"Log de Alterações do Cardápio"
     	array (
      	'strategy' => 'BIRT',
      	'parameterMapping' =>
	      	array (
	        	"DTMESCARD"		=>	"P_DTMESCARD",
			 	"NRLISTA"		=>	"P_NRLISTA",
			 	"NMTIPOPRAT"	=>	"P_CDTIPOPRAT",
			 	"CDPICTPRAT"    =>  "P_CDPICTPRAT",
				"NRORG"         =>  "P_NRORG"
			)
		)
	);
	public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
		$row              = $request->getRow();
		$cdfilial         = $row['NMFILIAL'] == 'T' ? 'T' : join("_", $row['NMFILIAL']);
		$cdgrupo          = $row['CDGRUPFILI'] ? $row['CDGRUPFILI'] : null;
		$row['DTMESCARD'] = "01/".$row['DTMESCARD'];
		$row['NMTIPOPRAT']= $row['NMTIPOPRAT']=="T" ? "T" : join("_", $row['NMTIPOPRAT']);
		
		$this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
		
		if($row['TSERVICO'] == 'S'){
			$nrLista          = $this->novoCodigo->getCodeByTableName('LISTA', $this->environment->getNrOrg());
			$row['NRLISTA']   = $nrLista;
			$this->insereListaSemServicoDefinido($nrLista, $cdfilial, $cdgrupo);
		}else{
			$row['NRLISTA'] = $this->utilPla->insertLista($row['NMSERVICO']);
		}
		$row['NRORG'] = $this->environment->getNrOrgTrab();
		
	}

	public function createRemoteReport(DTO\Request\Row $request, DTO\Response $response) {
		try {
	    	$teste = $this->beforeProcessReport($request, $response);
	    	$reportURL = $this->processReport($request);
			
			$this->afterProcessReport($request, $response, $reportURL);
			/** @TODO temporary solution */
			$response->addMessage(new DTO\Response\Message($reportURL));
			/** @TODO $response->open | download | show($report); */
		} catch (\Exception $e) {
			$response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
		} finally{
			$row = $request->getRow();
		}
	}
	
	protected function insereListaSemServicoDefinido($nrLista, $cdfilial, $cdgrupo){
		$sql = "INSERT INTO LISTA
				   (NRLISTA, CDLISTA)
				   (SELECT :P_NRLISTA, SERVICO.CDFILIAL||SERVOPER.CDSERVFILI AS CDFILISERV
						FROM SERVOPER, SERVICO, (SELECT F.CDFILIAL
													FROM FILIAL F, FILGRUPFILI G
													WHERE (G.CDGRUPFILI = :P_CDGRUPO) 
													AND (F.CDFILIAL = G.CDFILIAL)
												UNION ALL
												SELECT F.CDFILIAL
													FROM FILIAL F
													WHERE (:P_CDGRUPO IS NULL)) FILIAL, FILIOPER FP
						WHERE SERVOPER.CDOPERADOR = :P_CDOPERADOR
						AND ((INSTR(:P_CDFILIAL, SERVICO.CDFILIAL) <> 0) OR (:P_CDFILIAL = 'T'))
						AND SERVICO.CDFILIAL   = SERVOPER.CDFILIAL
						AND SERVICO.CDFILIAL   = FILIAL.CDFILIAL
						AND SERVOPER.CDFILIAL  = FILIAL.CDFILIAL
						AND SERVICO.CDSERVFILI = SERVOPER.CDSERVFILI
						AND SERVICO.IDTIPOSERV <> 'A'
						AND SERVICO.IDSERVATIV <> 'N'
						AND ((SERVICO.IDTPSERVHOSP IS NULL) OR (SERVICO.IDTPSERVHOSP = '0'))
						AND FILIAL.CDFILIAL = FP.CDFILIAL
						AND FP.CDOPERADOR = :P_CDOPERADOR
						UNION ALL
						SELECT :P_NRLISTA, SERVICO.CDFILIAL||SERVICO.CDSERVFILI AS CDFILISERV
						FROM SERVICO, (SELECT F.CDFILIAL
										FROM FILIAL F, FILGRUPFILI G
										WHERE (G.CDGRUPFILI = :P_CDGRUPO) 
											AND (F.CDFILIAL = G.CDFILIAL)
										UNION ALL
										SELECT F.CDFILIAL
										FROM FILIAL F
										WHERE (:P_CDGRUPO IS NULL)) FILIAL, FILIOPER FP
						WHERE ((INSTR(:P_CDFILIAL, SERVICO.CDFILIAL) <> 0) OR (:P_CDFILIAL = 'T'))
						AND IDTIPOSERV <> 'A'
						AND IDSERVATIV <> 'N'
						AND ((IDTPSERVHOSP IS NULL) OR (IDTPSERVHOSP = '0'))
						AND SERVICO.CDFILIAL = FILIAL.CDFILIAL
						AND FP.CDFILIAL = FILIAL.CDFILIAL
						AND FP.CDOPERADOR = :P_CDOPERADOR
						AND NOT EXISTS (SELECT SERVOPER.CDFILIAL
											FROM SERVOPER
											WHERE CDOPERADOR = :P_CDOPERADOR
											AND SERVOPER.CDFILIAL = SERVICO.CDFILIAL))";
    	try{
            $parameter = array(
                'P_NRLISTA'   => $nrLista,
                'P_CDFILIAL'  => $cdfilial,
                'P_CDGRUPO'   => $cdgrupo,
                'P_CDOPERADOR'=> $this->environment->getCdOperador()
            );
            $query = $this->conn->prepare($sql);
            $query->execute($parameter);
        } catch (\Exception $e) {
            throw $e;
        }
	}
	
	protected function insereListaComServico($nrLista, $servico){
		$sql = "INSERT INTO LISTA
				   (NRLISTA, CDLISTA)
				   SELECT :P_NRLISTA, CDFILIAL||CDSERVFILI
				      FROM SERVICO
				     WHERE VALOR_NA_LISTA(CDFILIAL||CDSERVFILI, :P_LISTASERV) <> 0";
		try{
            $parameter = array(
                'P_NRLISTA'    => $nrLista,
                'P_LISTASERV'  => $servico
            );
            $query = $this->conn->prepare($sql);
            $query->execute($parameter);
        } catch (\Exception $e) {
            throw $e;
        }
	}
	
	protected function deleteLista($nrLista){
		$sql = "DELETE FROM LISTA WHERE NRLISTA = :P_NRLISTA";
	    try{
            $parameter = array(
                'P_NRLISTA'     => $nrLista
            );
            $query = $this->conn->prepare($sql);
            $query->execute($parameter);
        } catch (\Exception $e) {
            throw $e;
        }
	}
}