<?php
namespace Teknisa\PLA\Services;

use Teknisa\Libs\Util\XLS\FileLoader;
use Teknisa\Libs\Util\XLS\FileReader;
use Teknisa\Libs\Util\XLS\Table;
use Teknisa\Libs\Util\XLS\Exception;
use Teknisa\Libs\Util\XLS\Filter;
use PHPExcel_Cell;

use Teknisa\PLA\Modules\Prato\Services\PratoService;

use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\FilterCriteria;
use \Setforms2\Shared\ORM\DateTime;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DTO;
use Doctrine\DBAL\Connection;

class Pla90000PratoEquivalenteService {
	
	protected $crudService;
	protected $parameterBag;
	protected $entityManager;
	protected $environment;
	protected $dataSourceManager;
	protected $util;
	protected $newCode;
	protected $connection;
	protected $novoCodigo;
	protected $prato;
    
  public function __construct($crudService, ParameterBag $parameterBag, $environment, $entityManager, $dataSourceManager, $util, $newCode, $connection, $novoCodigo, PratoService $prato) {
	    $this->crudService = $crudService;
	    $this->entityManager = $entityManager;
	    $this->environment = $environment;
	    $this->dataSourceManager = $dataSourceManager;
	    $this->util = $util;
	    $this->newCode = $newCode;
	    $this->connection = $connection;    
	    $this->novoCodigo = $novoCodigo;
	    $this->prato = $prato;
  }
	
	public function retornaPratoEquivalente($nrEquiporato){
		$query  = 
				"SELECT 
					PE.NREQUIPRATO, 
					PE.NRPRATEQUIPRATO, 
					PE.CDFILAUXPRAT, 
					PE.CDPRATO, 
					P.NMPRATO
				FROM 
					PRATEQUIPRATO PE, 
					PRATO P
				WHERE 
					PE.CDFILAUXPRAT = P.CDFILAUXPRAT
					AND PE.CDPRATO = P.CDPRATO
					AND PE.NREQUIPRATO = :P_NREQUIPRATO";

		$params = array('P_NREQUIPRATO' => $nrEquiporato);

		$sql = $this->entityManager->getConnection()->prepare($query);
        return $this->entityManager->getConnection()->fetchAll($query, $params);
	}
	
	public function insertPratoPrincipal($rows){
		$query = 
				"INSERT INTO 
					EQUIPRATO(
						NREQUIPRATO, 
						CDFILAUXPRAT, 
						CDPRATO, 
						IDEQUIPRATO,
						CDFILIAL, 
						CDSERVFILI
					)
				VALUES (
						SEQ_EQUIPRATO.NEXTVAL, 
						:CDFILAUXPRAT, 
						:CDPRATO, 
						:IDEQUIPRATO, 
						:CDFILIAL, 
						:CDSERVFILI
					)";
		$params = array(
            "CDFILAUXPRAT"  => $rows['CDFILAUXPRAT'],
            "CDPRATO"				=> $rows['CDPRATO'],
            "IDEQUIPRATO"		=> $rows['IDEQUIPRATO'],
            "CDFILIAL"			=> $rows['CDFILIAL'],
            "CDSERVFILI"		=> $rows['CDSERVFILI']
        );
        
        $std = $this-> connection->prepare($query);
        $std->execute($params);
	}
	
	public function insertPratoEquivalente($rows){
		$query = 
				"INSERT INTO 
					PRATEQUIPRATO(
						NRPRATEQUIPRATO, 
						NREQUIPRATO, 
						CDFILAUXPRAT,
						CDPRATO
					)
				VALUES (
						SEQ_PRATEQUIPRATO.NEXTVAL, 
						:NREQUIPRATO, 
						:CDFILAUXPRAT, 
						:CDPRATO
					)";
				
		$params = array(
            "NREQUIPRATO"		=> $rows['NREQUIPRATO'],
            "CDFILAUXPRAT"	=> $rows['CDFILAUXPRAT'],
            "CDPRATO"				=> $rows['CDPRATO']
        );
        
        $std = $this->entityManager->getConnection()->prepare($query);
        $std->execute($params);
	}
	
	public function deletaPratoEquivalente($rows){
		$query = 
				"DELETE FROM PRATEQUIPRATO 
				 WHERE
					CDFILAUXPRAT	=	:CDFILAUXPRAT AND
					CDPRATO			=	:CDPRATO AND
					NREQUIPRATO		=   :NREQUIPRATO AND
					NRPRATEQUIPRATO =	:NRPRATEQUIPRATO";
					
		$params = array(
            "CDFILAUXPRAT"		=> $rows['CDFILAUXPRAT'],
            "CDPRATO"			=> $rows['CDPRATO'],
            "NREQUIPRATO"		=> $rows['NREQUIPRATO'],
            "NRPRATEQUIPRATO"	=> $rows['NRPRATEQUIPRATO']
        );
        
        $std = $this->entityManager->getConnection()->prepare($query);
        $std->execute($params);
	}
	
	public function retornaPratoParaInclusao($row){
		$query = 
			"SELECT P.NRORG, 
				P.CDPRATO, 
				P.CDFILAUXPRAT, 
				P.NMPRATO 
			FROM 
				PRATO P 
			WHERE 
				((P.CDFILAUXPRAT = 'P') OR (P.CDFILAUXPRAT = :P_CDFILIAL)) 
				AND (P.CDNVPRATO = :P_CDNVPRATO)";
		
		$params = array(
            "P_CDNVPRATO"	=> $row['CDNVPRATO'],
            "P_CDFILIAL"	=> $row['CDFILIAL']
        );
		
		$sql = $this->entityManager->getConnection()->prepare($query);
        return $this->entityManager->getConnection()->fetchAll($query, $params);
	}
	
	public function readXls($row) {
        $file = $row['FILE'];
        $columnNames = $row['CAMPOS'];
        $sheetsToLoad = array('Sheasdasdsadet1');
        $xlsFile = FileLoader::loadFromBase64($file['content'], $file['name']);
        
        if(!$xlsFile){
            throw new \Exception(self::IMPORT_LOAD_FILE_ERROR);
        }
        
        $worksheet = $xlsFile->getActiveSheet();
        $table = FileReader::readWorksheetTable($worksheet, $columnNames);
        
        return $table;		
	}
	
	public function formatXls(Table $table, $tabela) {
        $columnsReference = $table->getColumnsReference();
        $tableRows = $table->getRows();	
        $columnsNumber = $table->getNumberOfColumns();
        $i = 0;
        $erros = array();
        $rowAnt = array();
        $produtos = array();
        $cdpictprat = $this->prato->getCdPictPrat();
        
        foreach ($tableRows as $chave => &$row) {
						$chave = $chave+2;
            $row = $row['content'];
            $originalRow = new \ArrayObject($row);
            $originalRow = $originalRow->getArrayCopy(); 
            
            for($column = 0; $column < $columnsNumber; $column++){
                $columnLetterIndex = PHPExcel_Cell::stringFromColumnIndex($column);
                $columnName = $columnsReference[$columnLetterIndex];  
                $isCompleteLevelPrato = $this->prato->hasCompleteLevel($row['CDPRATO'], $cdpictprat);
                $currentPlate = $row['CDPRATO'];
                $isCompleteLevelPratoDest = $this->prato->hasCompleteLevel($row['CDPRATODEST'], $cdpictprat);
                
								try {
								  switch($columnName){
								  	case 'CDFILAUXPRAT':
							  			if($row[$columnName] == NULL) {
							  				$erros[$chave][] = "CDFILAUXPRAT_NAO_INFORMADO"; //retornou um objeto
							  			}
								
								  		break;
					 					case 'CDPRATO':
								 			if($row[$columnName]) {
								   			if(!$isCompleteLevelPrato) {
								   				$erros[$chave][] = "CDPRATO_NIVEL_INCOMPLETO";
								   			}
								   			$pratoParaincInclusao = $this->verificaPrato($row["$columnName"], $row["CDFILAUXPRAT"]);
								 				if($pratoParaincInclusao == false) {
								 					$erros[$chave][] = "CDPRATO_NAO_CADASTRADO";
								 				}
								 			}
								 			if($row[$columnName] == NULL) {
								 				$erros[$chave][] = "CDPRATO_PRINCIPAL_NAO_INFORMADO";
								 			}
								 			break;
							 			case 'CDFILIAL':
								 			if($row[$columnName]) {
								 				$existeFilialOper = $this->verificaExisteFilialOper($row[$columnName]);
								 		  	if(count($existeFilialOper) == 0) { //verificar se CDFILIAL existe na base
								 		  		$erros[$chave][] = "UNIDADE_NAO_CADASTRADA";
								 		  	}
								 			}
								  			if($row['CDFILAUXPRAT'] != 'P' && $row['CDFILAUXPRAT'] != NULL) {
								  				if($row[$columnName] == NULL) {
								  					$erros[$chave][] = "PRATO_PRINCIPAL_LOCAL_INFORMADO_MAS_UNIDADE_NAO_INFORMADA";
								  				} else if($row[$columnName] != $row['CDFILAUXPRAT']) {
								 	 				$erros[$chave][] = "PRATO_PRINCIPAL_LOCAL_INFORMADO_NAO_PERTENCE_A_UNIDADE_INFORMADA";
								 	 			}
								  			}
								 			break;
								 		case 'CDSERVFILI':
								 			if($row["CDFILIAL"]) {
								 				if($row[$columnName] != NULL) {
								   				$existeServico = $this->verificaServicoExiste($row["CDFILIAL"], $row[$columnName]);
								   				if(count($existeServico) == 0) {
								   					$erros[$chave][] = "SERVICO_INFORMADO_NAO_CADASTRADO";
								   				}
								 				}
								 			}
								 			if($row[$columnName]) {
									 			if($row["CDFILIAL"] == NULL) {
									 				$erros[$chave][] = "SERVICO_INFORMADO_SEM_UNIDADE";
									 			}
								 			}
								 			break;
								 		case 'CDFILAUXPRATDEST':
											if($row[$columnName] == NULL) {
												$erros[$chave][] = "CDFILAUXPRATDEST_NAO_INFORMADO";
											}
											if($row[$columnName] != 'P' && $row[$columnName] != NULL) {
												if($row["CDFILIAL"] == NULL) {
													$erros[$chave][] = "PRATO_EQUIV_LOCAL_INFORMADO_SEM_UNIDADE_INFORMADA";
												} else if($row["CDFILIAL"] != $row["CDFILAUXPRATDEST"]) {
													$erros[$chave][] = "PRATO_EQUIV_INFORMADO_NAO_PERTENCE_A_UNIDADE_INFORMADA";
												}
											}
								 			break;
								 		case 'CDPRATODEST':
								 		if($row[$columnName] == NULL) {
								 				$erros[$chave][] = "PRATO_EQUIVALENTE_NAO_INFORMADO";
								 		}
								 		if(!$isCompleteLevelPratoDest) {
								 			$erros[$chave][] = "CDPRATODEST_NIVEL_INCOMPLETO";
								 		} 
								 		
										$pratoDestParaincInclusao = $this->verificaPrato($row["$columnName"], $row["CDFILAUXPRATDEST"]);
										if($pratoDestParaincInclusao == false) {
											$erros[$chave][] = "CDPRATOEQUI_NAO_CADASTRADO";
										}
								 		break;
								  }
								}catch (\Exception $e) {
								  $lista_erros = json_decode($e->getMessage());
								}
            }
            
            $row["ROWNUM"] = $i;
            $i++;            
        }
        
        if(count($erros) != 0) {
        	return array('ROWS'=>$tableRows, 'ERROR'=>$erros);
        }
        $rowsToInsert = array('ROWS'=>$tableRows);
        $rows = $rowsToInsert["ROWS"];
        $registrosSalvos = 0;
        
        foreach($rows as $item) {
					$idEquiprato = 'P';
					
					if($item["CDFILIAL"]) {
						$idEquiprato = 'F';
					}
					if($item["CDFILIAL"] && $item["CDSERVFILI"]) {
						$idEquiprato = 'S';
					}
					
        	$paramsPratoPrincipal = [
        		"CDFILAUXPRAT"	=> $item["CDFILAUXPRAT"], 
						"CDPRATO"				=> $item["CDPRATO"], 
						"IDEQUIPRATO"	  => $idEquiprato, 
						"CDFILIAL"			=> $item["CDFILIAL"],
						"CDSERVFILI"		=> $item["CDSERVFILI"]
        	];
        	
        	$nrEquiprato = $this->getNrEquiPrato($paramsPratoPrincipal);
        	
        	if($nrEquiprato == false) {
        		$this->insertPratoPrincipal($paramsPratoPrincipal);
        		$nrEquiprato = $this->getNrEquiPrato($paramsPratoPrincipal);
        	}
        	
        	
        	$item["NREQUIPRATO"] = $nrEquiprato;
        	
        	$paramsPratoEquiv = [
        		"NREQUIPRATO" => $nrEquiprato, 
						"CDFILAUXPRAT" => $item["CDFILAUXPRATDEST"], 
						"CDPRATO"	=> $item["CDPRATODEST"]
        	];
        	
        	if($this->existePratoEquivalente($paramsPratoEquiv) == false) {
        		$this->insertPratoEquivalente($paramsPratoEquiv);
        	}
        	
        	$registrosSalvos++;
        }
        
        return ["SUCCESS" => $registrosSalvos];
	}
	
	public function existePratoEquivalente($params) {
		$query = "
			SELECT NREQUIPRATO
			FROM PRATEQUIPRATO
			WHERE 
			NREQUIPRATO = :NREQUIPRATO 
			AND CDFILAUXPRAT = :CDFILAUXPRAT
			AND CDPRATO = :CDPRATO
		";
		
		$sql = $this->entityManager->getConnection()->prepare($query);
   	$sql->execute($params);
    $pratEquiPrato = $sql->fetch();
    if($pratEquiPrato != false) {
    	return true;
    } 
    
    return false;
		
	}
	
	public function getNrEquiPrato($row) {
		$params = [];
		
		if($row["IDEQUIPRATO"] == "P") {
			$query = "
				SELECT NREQUIPRATO
			  FROM EQUIPRATO
				WHERE CDFILAUXPRAT = :CDFILAUXPRAT
			   AND CDPRATO = :CDPRATO
			   AND IDEQUIPRATO = 'P'
			";
			
			$params = [
				"CDFILAUXPRAT" => $row["CDFILAUXPRAT"],
				"CDPRATO"	=>	$row["CDPRATO"]
			];
		}
		
		if($row["IDEQUIPRATO"] == "F") {
			$query = "
				SELECT NREQUIPRATO
				FROM EQUIPRATO
				WHERE CDFILAUXPRAT = :CDFILAUXPRAT
				AND CDPRATO = :CDPRATO
				AND IDEQUIPRATO = 'F'
				AND CDFILIAL = :CDFILIAL
			";
			
			$params = [
				"CDFILAUXPRAT" => $row["CDFILAUXPRAT"],
				"CDPRATO"	=>	$row["CDPRATO"],
				"CDFILIAL" => $row["CDFILIAL"]
			];
		}
		
		if($row["IDEQUIPRATO"] == "S") {
			$query = "
				SELECT NREQUIPRATO
				FROM EQUIPRATO
				WHERE CDFILAUXPRAT = :CDFILAUXPRAT
				AND CDPRATO = :CDPRATO
				AND IDEQUIPRATO = 'S'
				AND CDFILIAL = :CDFILIAL
				AND CDSERVFILI = :CDSERVFILI
			";			
			
			$params = [
				"CDFILAUXPRAT" => $row["CDFILAUXPRAT"],
				"CDPRATO"	=>	$row["CDPRATO"],
				"CDFILIAL" => $row["CDFILIAL"],
				"CDSERVFILI" => $row["CDSERVFILI"]
			];
		}
		   
   	$sql = $this->entityManager->getConnection()->prepare($query);
   	$sql->execute($params);
    $equiprato = $sql->fetch();
    if($equiprato != false) {
    	return $equiprato["NREQUIPRATO"];
    } 
    
    return false;
	}
	
	public function verificaExisteFilialOper($cdFilial) {
			$query = "
				SELECT DISTINCT
				    F.NRORG,
				    F.CDFILIAL,
				    F.NMFILIAL,
				    CASE
				        WHEN NVL(IDUTILDTPROC, 'N') = 'N' THEN
				            TO_CHAR(SYSDATE, 'DD/MM/YYYY')
				        ELSE
				            TO_CHAR(NVL(PF.DTPROCESSA, SYSDATE),
				                    'DD/MM/YYYY')
				    END DTPROCESSA,
				    PF.IDPERMESTNEG,
				    PF.IDUTILLOTE,
				    PF.IDUTILALMOX
				FROM
				    PARAMGERAL  PG, FILIAL F
				    INNER JOIN FILIOPER    FO ON FO.NRORG = F.NRORG
				                              AND FO.CDFILIAL = F.CDFILIAL
				    LEFT JOIN PARAMFILIAL PF ON PF.NRORG = F.NRORG
				                                AND PF.CDFILIAL = F.CDFILIAL
				WHERE
				        F.NRORG = :NRORG
				    AND FO.CDOPERADOR = :CDOPERADOR	
				    AND F.CDFILIAL = :CDFILIAL
			";
		
		$params = [
			"CDOPERADOR"	=> $this->environment->getCdOperador(),
			"NRORG" 			=> $this->environment->getNrOrgTrab(),
			"CDFILIAL"		=> $cdFilial
		];
		
		$sql = $this->entityManager->getConnection()->prepare($query);
		return $this->entityManager->getConnection()->fetchAll($query, $params);
  }
  
  public function verificaServicoExiste($cdFilial, $cdServFili) {
  	$query = "SELECT CDFILIAL, CDSERVFILI, NMSERVICO FROM SERVICO WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI AND NRORG = :NRORG";
  	$params = [
			"CDFILIAL"		=> $cdFilial,
			"CDSERVFILI"	=> $cdServFili,
			"NRORG" 			=> $this->environment->getNrOrgTrab()
  	];
  	
  	$sql = $this->entityManager->getConnection()->prepare($query);
		return $this->entityManager->getConnection()->fetchAll($query, $params);
  	
  }
  
  public function verificaPrato($cdPrato, $cdFilAuxPrat) {
  	$query = "
			SELECT
				COUNT(*) AS PRATO
			FROM
				PRATO P
			WHERE				
				P.CDPRATO = :P_CDPRATO AND 
				P.CDFILAUXPRAT = :P_CDFILAUXPRAT
  	";

  	$params = [
			"P_CDPRATO"		=> $cdPrato,
			"P_CDFILAUXPRAT" => $cdFilAuxPrat
		];
  	
  	$sql = $this->entityManager->getConnection()->prepare($query);
	$retorno = $this->entityManager->getConnection()->fetchAll($query, $params);  	
	if(count($retorno) > 0){
		if($retorno[0]["PRATO"] > 0)
			return true;
	}
	return false;
  }
}